@@ -12,6 +12,7 @@
 #include <cstdint>
 
 #include "xsimd_base.hpp"
+#include "xsimd_int_conversion.hpp"
 
 namespace xsimd
 {
@@ -42,23 +43,13 @@ namespace xsimd
 
         operator __m256i() const;
 
+        bool operator[](std::size_t index) const;
+
     private:
 
         __m256i m_value;
     };
 
-    batch_bool<int32_t, 8> operator&(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> operator|(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> operator^(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> operator~(const batch_bool<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> bitwise_andnot(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs);
-
-    batch_bool<int32_t, 8> operator==(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> operator!=(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs);
-
-    bool all(const batch_bool<int32_t, 8>& rhs);
-    bool any(const batch_bool<int32_t, 8>& rhs);
-
     /*********************
      * batch<int32_t, 8> *
      *********************/
@@ -100,6 +91,12 @@ namespace xsimd
         batch& load_aligned(const double* src);
         batch& load_unaligned(const double* src);
 
+        batch& load_aligned(const char* src);
+        batch& load_unaligned(const char* src);
+
+        batch& load_aligned(const unsigned char* src);
+        batch& load_unaligned(const unsigned char* src);
+
         void store_aligned(int32_t* dst) const;
         void store_unaligned(int32_t* dst) const;
 
@@ -112,6 +109,12 @@ namespace xsimd
         void store_aligned(double* dst) const;
         void store_unaligned(double* dst) const;
 
+        void store_aligned(char* dst) const;
+        void store_unaligned(char* dst) const;
+
+        void store_aligned(unsigned char* dst) const;
+        void store_unaligned(unsigned char* dst) const;
+
         int32_t operator[](std::size_t index) const;
 
     private:
@@ -119,37 +122,6 @@ namespace xsimd
         __m256i m_value;
     };
 
-    batch<int32_t, 8> operator-(const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> operator+(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> operator-(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> operator*(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> operator/(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-
-    batch_bool<int32_t, 8> operator==(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> operator!=(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> operator<(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch_bool<int32_t, 8> operator<=(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-
-    batch<int32_t, 8> operator&(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> operator|(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> operator^(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> operator~(const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> bitwise_andnot(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-
-    batch<int32_t, 8> min(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-    batch<int32_t, 8> max(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs);
-
-    batch<int32_t, 8> abs(const batch<int32_t, 8>& rhs);
-
-    batch<int32_t, 8> fma(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z);
-    batch<int32_t, 8> fms(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z);
-    batch<int32_t, 8> fnma(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z);
-    batch<int32_t, 8> fnms(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z);
-
-    int32_t hadd(const batch<int32_t, 8>& rhs);
-
-    batch<int32_t, 8> select(const batch_bool<int32_t, 8>& cond, const batch<int32_t, 8>& a, const batch<int32_t, 8>& b);
-
     batch<int32_t, 8> operator<<(const batch<int32_t, 8>& lhs, int32_t rhs);
     batch<int32_t, 8> operator>>(const batch<int32_t, 8>& lhs, int32_t rhs);
 
@@ -205,76 +177,92 @@ namespace xsimd
         return m_value;
     }
 
-    inline batch_bool<int32_t, 8> operator&(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs)
+    inline bool batch_bool<int32_t, 8>::operator[](std::size_t index) const
     {
+        alignas(32) int32_t x[8];
+        _mm256_store_si256((__m256i*)x, m_value);
+        return static_cast<bool>(x[index & 7]);
+    }
+
+    namespace detail
+    {
+        template <>
+        struct batch_bool_kernel<int32_t, 8>
+        {
+            using batch_type = batch_bool<int32_t, 8>;
+
+            static batch_type bitwise_and(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_and_si256(lhs, rhs);
+                return _mm256_and_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_and_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_and_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> operator|(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_or(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_or_si256(lhs, rhs);
+                return _mm256_or_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_or_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_or_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> operator^(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_xor(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_xor_si256(lhs, rhs);
+                return _mm256_xor_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_xor_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_xor_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> operator~(const batch_bool<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_not(const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_xor_si256(rhs, _mm256_set1_epi32(-1));
+                return _mm256_xor_si256(rhs, _mm256_set1_epi32(-1));
 #else
-        XSIMD_SPLIT_AVX(rhs);
-        __m128i res_low = _mm_xor_si128(rhs_low, _mm_set1_epi32(-1));
-        __m128i res_high = _mm_xor_si128(rhs_high, _mm_set1_epi32(-1));
-        XSIMD_RETURN_MERGED_SSE(res_low, res_high);
+                XSIMD_SPLIT_AVX(rhs);
+                __m128i res_low = _mm_xor_si128(rhs_low, _mm_set1_epi32(-1));
+                __m128i res_high = _mm_xor_si128(rhs_high, _mm_set1_epi32(-1));
+                XSIMD_RETURN_MERGED_SSE(res_low, res_high);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> bitwise_andnot(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_andnot(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_andnot_si256(lhs, rhs);
+                return _mm256_andnot_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_andnot_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_andnot_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> operator==(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs)
-    {
+            static batch_type equal(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_cmpeq_epi32(lhs, rhs);
+                return _mm256_cmpeq_epi32(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_cmpeq_epi32, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_cmpeq_epi32, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> operator!=(const batch_bool<int32_t, 8>& lhs, const batch_bool<int32_t, 8>& rhs)
-    {
-        return ~(lhs == rhs);
-    }
+            static batch_type not_equal(const batch_type& lhs, const batch_type& rhs)
+            {
+                return ~(lhs == rhs);
+            }
 
-    inline bool all(const batch_bool<int32_t, 8>& rhs)
-    {
-        return _mm256_testc_si256(rhs, batch_bool<int32_t, 8>(true)) != 0;
-    }
+            static bool all(const batch_type& rhs)
+            {
+                return _mm256_testc_si256(rhs, batch_bool<int32_t, 8>(true)) != 0;
+            }
 
-    inline bool any(const batch_bool<int32_t, 8>& rhs)
-    {
-        return !_mm256_testz_si256(rhs, rhs);
+            static bool any(const batch_type& rhs)
+            {
+                return !_mm256_testz_si256(rhs, rhs);
+            }
+        };
     }
 
     /************************************
@@ -341,14 +329,9 @@ namespace xsimd
     inline batch<int32_t, 8>& batch<int32_t, 8>::load_aligned(const int64_t* src)
     {
         alignas(32) int32_t tmp[8];
-        tmp[0] = static_cast<int32_t>(src[0]);
-        tmp[1] = static_cast<int32_t>(src[1]);
-        tmp[2] = static_cast<int32_t>(src[2]);
-        tmp[3] = static_cast<int32_t>(src[3]);
-        tmp[4] = static_cast<int32_t>(src[4]);
-        tmp[5] = static_cast<int32_t>(src[5]);
-        tmp[6] = static_cast<int32_t>(src[6]);
-        tmp[7] = static_cast<int32_t>(src[7]);
+        unroller<8>([&](std::size_t i){
+            tmp[i] = static_cast<int32_t>(src[i]);
+        });
         m_value = _mm256_load_si256((__m256i const*)tmp);
         return *this;
     }
@@ -388,6 +371,30 @@ namespace xsimd
         return *this;
     }
 
+    inline batch<int32_t, 8>& batch<int32_t, 8>::load_aligned(const char* src)
+    {
+        __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
+        m_value = detail::xsimd_cvtepi8_epi32(tmp);
+        return *this;
+    }
+
+    inline batch<int32_t, 8>& batch<int32_t, 8>::load_unaligned(const char* src)
+    {
+        return load_aligned(src);
+    }
+
+    inline batch<int32_t, 8>& batch<int32_t, 8>::load_aligned(const unsigned char* src)
+    {
+        __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
+        m_value = detail::xsimd_cvtepu8_epi32(tmp);
+        return *this;
+    }
+
+    inline batch<int32_t, 8>& batch<int32_t, 8>::load_unaligned(const unsigned char* src)
+    {
+        return load_aligned(src);
+    }
+
     inline void batch<int32_t, 8>::store_aligned(int32_t* dst) const
     {
         _mm256_store_si256((__m256i*)dst, m_value);
@@ -402,14 +409,9 @@ namespace xsimd
     {
         alignas(32) int32_t tmp[8];
         store_aligned(tmp);
-        dst[0] = int64_t(tmp[0]);
-        dst[1] = int64_t(tmp[1]);
-        dst[2] = int64_t(tmp[2]);
-        dst[3] = int64_t(tmp[3]);
-        dst[4] = int64_t(tmp[4]);
-        dst[5] = int64_t(tmp[5]);
-        dst[6] = int64_t(tmp[6]);
-        dst[7] = int64_t(tmp[7]);
+        unroller<8>([&](std::size_t i) {
+            dst[i] = static_cast<int64_t>(tmp[i]);
+        });
     }
 
     inline void batch<int32_t, 8>::store_unaligned(int64_t* dst) const
@@ -443,6 +445,28 @@ namespace xsimd
         _mm256_storeu_pd(dst + 4, _mm256_cvtepi32_pd(tmp2));
     }
 
+    inline void batch<int32_t, 8>::store_aligned(char* dst) const
+    {
+        __m128i tmp = detail::xsimd_cvtepi32_epi8(m_value);
+        _mm_storel_epi64((__m128i*)dst, tmp);
+    }
+
+    inline void batch<int32_t, 8>::store_unaligned(char* dst) const
+    {
+        store_aligned(dst);
+    }
+
+    inline void batch<int32_t, 8>::store_aligned(unsigned char* dst) const
+    {
+        __m128i tmp = detail::xsimd_cvtepi32_epu8(m_value);
+        _mm_storel_epi64((__m128i*)dst, tmp);
+    }
+
+    inline void batch<int32_t, 8>::store_unaligned(unsigned char* dst) const
+    {
+        store_aligned(dst);
+    }
+
     inline int32_t batch<int32_t, 8>::operator[](std::size_t index) const
     {
         alignas(32) int32_t x[8];
@@ -450,205 +474,237 @@ namespace xsimd
         return x[index & 7];
     }
 
-    inline batch<int32_t, 8> operator-(const batch<int32_t, 8>& rhs)
+    namespace detail
     {
+        template <>
+        struct batch_kernel<int32_t, 8>
+        {
+            using batch_type = batch<int32_t, 8>;
+            using value_type = int32_t;
+            using batch_bool_type = batch_bool<int32_t, 8>;
+
+            static batch_type neg(const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_sub_epi32(_mm256_setzero_si256(), rhs);
+                return _mm256_sub_epi32(_mm256_setzero_si256(), rhs);
 #else
-        XSIMD_SPLIT_AVX(rhs);
-        __m128i res_low = _mm_sub_epi32(_mm_setzero_si128(), rhs_low);
-        __m128i res_high = _mm_sub_epi32(_mm_setzero_si128(), rhs_high);
-        XSIMD_RETURN_MERGED_SSE(res_low, res_high);
+                XSIMD_SPLIT_AVX(rhs);
+                __m128i res_low = _mm_sub_epi32(_mm_setzero_si128(), rhs_low);
+                __m128i res_high = _mm_sub_epi32(_mm_setzero_si128(), rhs_high);
+                XSIMD_RETURN_MERGED_SSE(res_low, res_high);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> operator+(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type add(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_add_epi32(lhs, rhs);
+                return _mm256_add_epi32(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_add_epi32, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_add_epi32, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> operator-(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type sub(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_sub_epi32(lhs, rhs);
+                return _mm256_sub_epi32(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_sub_epi32, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_sub_epi32, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> operator*(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type mul(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_mullo_epi32(lhs, rhs);
+                return _mm256_mullo_epi32(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_mullo_epi32, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_mullo_epi32, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> operator/(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
-        return _mm256_cvttps_epi32(_mm256_div_ps(_mm256_cvtepi32_ps(lhs), _mm256_cvtepi32_ps(rhs)));
-    }
+            static batch_type div(const batch_type& lhs, const batch_type& rhs)
+            {
+#if defined(XSIMD_FAST_INTEGER_DIVISION)
+                return _mm256_cvttps_epi32(_mm256_div_ps(_mm256_cvtepi32_ps(lhs), _mm256_cvtepi32_ps(rhs)));
+#else
+                alignas(64) int32_t tmp_lhs[8], tmp_rhs[8], tmp_res[8];
+                lhs.store_aligned(tmp_lhs);
+                rhs.store_aligned(tmp_rhs);
+                unroller<8>([&](std::size_t i) {
+                    tmp_res[i] = tmp_lhs[i] / tmp_rhs[i];
+                });
+                return batch_type(tmp_res, aligned_mode());
+#endif
+            }
 
-    inline batch_bool<int32_t, 8> operator==(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type mod(const batch_type& lhs, const batch_type& rhs)
+            {
+                alignas(64) int32_t tmp_lhs[8], tmp_rhs[8], tmp_res[8];
+                lhs.store_aligned(tmp_lhs);
+                rhs.store_aligned(tmp_rhs);
+                unroller<8>([&](std::size_t i) {
+                    tmp_res[i] = tmp_lhs[i] % tmp_rhs[i];
+                });
+                return batch_type(tmp_res, aligned_mode());
+            }
+
+            static batch_bool_type eq(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_cmpeq_epi32(lhs, rhs);
+                return _mm256_cmpeq_epi32(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_cmpeq_epi32, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_cmpeq_epi32, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> operator!=(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
-        return ~(lhs == rhs);
-    }
+            static batch_bool_type neq(const batch_type& lhs, const batch_type& rhs)
+            {
+                return ~(lhs == rhs);
+            }
 
-    inline batch_bool<int32_t, 8> operator<(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_bool_type lt(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_cmpgt_epi32(rhs, lhs);
+                return _mm256_cmpgt_epi32(rhs, lhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_cmpgt_epi32, rhs, lhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_cmpgt_epi32, rhs, lhs);
 #endif
-    }
+            }
 
-    inline batch_bool<int32_t, 8> operator<=(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
-        return ~(rhs < lhs);
-    }
+            static batch_bool_type lte(const batch_type& lhs, const batch_type& rhs)
+            {
+                return ~(rhs < lhs);
+            }
 
-    inline batch<int32_t, 8> operator&(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_and(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_and_si256(lhs, rhs);
+                return _mm256_and_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_and_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_and_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> operator|(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_or(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_or_si256(lhs, rhs);
+                return _mm256_or_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_or_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_or_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> operator^(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_xor(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_xor_si256(lhs, rhs);
+                return _mm256_xor_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_xor_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_xor_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> operator~(const batch<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_not(const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_xor_si256(rhs, _mm256_set1_epi32(-1));
+                return _mm256_xor_si256(rhs, _mm256_set1_epi32(-1));
 #else
-        XSIMD_SPLIT_AVX(rhs);
-        __m128i res_low = _mm_xor_si128(rhs_low, _mm_set1_epi32(-1));
-        __m128i res_high = _mm_xor_si128(rhs_high, _mm_set1_epi32(-1));
-        XSIMD_RETURN_MERGED_SSE(res_low, res_high);
+                XSIMD_SPLIT_AVX(rhs);
+                __m128i res_low = _mm_xor_si128(rhs_low, _mm_set1_epi32(-1));
+                __m128i res_high = _mm_xor_si128(rhs_high, _mm_set1_epi32(-1));
+                XSIMD_RETURN_MERGED_SSE(res_low, res_high);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> bitwise_andnot(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type bitwise_andnot(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_andnot_si256(lhs, rhs);
+                return _mm256_andnot_si256(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_andnot_si128, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_andnot_si128, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> min(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type min(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_min_epi32(lhs, rhs);
+                return _mm256_min_epi32(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_min_epi32, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_min_epi32, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> max(const batch<int32_t, 8>& lhs, const batch<int32_t, 8>& rhs)
-    {
+            static batch_type max(const batch_type& lhs, const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_max_epi32(lhs, rhs);
+                return _mm256_max_epi32(lhs, rhs);
 #else
-        XSIMD_APPLY_SSE_FUNCTION(_mm_max_epi32, lhs, rhs);
+                XSIMD_APPLY_SSE_FUNCTION(_mm_max_epi32, lhs, rhs);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> abs(const batch<int32_t, 8>& rhs)
-    {
+            static batch_type abs(const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_sign_epi32(rhs, rhs);
+                return _mm256_sign_epi32(rhs, rhs);
 #else
-        XSIMD_SPLIT_AVX(rhs);
-        __m128i res_low = _mm_sign_epi32(rhs_low, rhs_low);
-        __m128i res_high = _mm_sign_epi32(rhs_high, rhs_high);
-        XSIMD_RETURN_MERGED_SSE(res_low, res_high);
+                XSIMD_SPLIT_AVX(rhs);
+                __m128i res_low = _mm_sign_epi32(rhs_low, rhs_low);
+                __m128i res_high = _mm_sign_epi32(rhs_high, rhs_high);
+                XSIMD_RETURN_MERGED_SSE(res_low, res_high);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> fma(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z)
-    {
-        return x * y + z;
-    }
+            static batch_type fma(const batch_type& x, const batch_type& y, const batch_type& z)
+            {
+                return x * y + z;
+            }
 
-    inline batch<int32_t, 8> fms(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z)
-    {
-        return x * y - z;
-    }
+            static batch_type fms(const batch_type& x, const batch_type& y, const batch_type& z)
+            {
+                return x * y - z;
+            }
 
-    inline batch<int32_t, 8> fnma(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z)
-    {
-        return -x * y + z;
-    }
+            static batch_type fnma(const batch_type& x, const batch_type& y, const batch_type& z)
+            {
+                return -x * y + z;
+            }
 
-    inline batch<int32_t, 8> fnms(const batch<int32_t, 8>& x, const batch<int32_t, 8>& y, const batch<int32_t, 8>& z)
-    {
-        return -x * y - z;
-    }
+            static batch_type fnms(const batch_type& x, const batch_type& y, const batch_type& z)
+            {
+                return -x * y - z;
+            }
 
-    inline int32_t hadd(const batch<int32_t, 8>& rhs)
-    {
+            static value_type hadd(const batch_type& rhs)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        __m256i tmp1 = _mm256_hadd_epi32(rhs, rhs);
-        __m256i tmp2 = _mm256_hadd_epi32(tmp1, tmp1);
-        __m128i tmp3 = _mm256_extracti128_si256(tmp2, 1);
-        __m128i tmp4 = _mm_add_epi32(_mm256_castsi256_si128(tmp2), tmp3);
-        return _mm_cvtsi128_si32(tmp4);
+                __m256i tmp1 = _mm256_hadd_epi32(rhs, rhs);
+                __m256i tmp2 = _mm256_hadd_epi32(tmp1, tmp1);
+                __m128i tmp3 = _mm256_extracti128_si256(tmp2, 1);
+                __m128i tmp4 = _mm_add_epi32(_mm256_castsi256_si128(tmp2), tmp3);
+                return _mm_cvtsi128_si32(tmp4);
 #else
-        XSIMD_SPLIT_AVX(rhs);
-        __m128i tmp1 = _mm_add_epi32(rhs_low, rhs_high);
-        __m128i tmp2 = _mm_hadd_epi32(tmp1, tmp1);
-        __m128i tmp3 = _mm_hadd_epi32(tmp2, tmp2);
-        return _mm_cvtsi128_si32(tmp3);
+                XSIMD_SPLIT_AVX(rhs);
+                __m128i tmp1 = _mm_add_epi32(rhs_low, rhs_high);
+                __m128i tmp2 = _mm_hadd_epi32(tmp1, tmp1);
+                __m128i tmp3 = _mm_hadd_epi32(tmp2, tmp2);
+                return _mm_cvtsi128_si32(tmp3);
 #endif
-    }
+            }
 
-    inline batch<int32_t, 8> select(const batch_bool<int32_t, 8>& cond, const batch<int32_t, 8>& a, const batch<int32_t, 8>& b)
-    {
+            static batch_type select(const batch_bool_type& cond, const batch_type& a, const batch_type& b)
+            {
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX2_VERSION
-        return _mm256_blendv_epi8(b, a, cond);
+                return _mm256_blendv_epi8(b, a, cond);
 #else
-        XSIMD_SPLIT_AVX(cond);
-        XSIMD_SPLIT_AVX(a);
-        XSIMD_SPLIT_AVX(b);
-        __m128i res_low = _mm_blendv_epi8(b_low, a_low, cond_low);
-        __m128i res_high = _mm_blendv_epi8(b_high, a_high, cond_high);
-        XSIMD_RETURN_MERGED_SSE(res_low, res_high);
+                XSIMD_SPLIT_AVX(cond);
+                XSIMD_SPLIT_AVX(a);
+                XSIMD_SPLIT_AVX(b);
+                __m128i res_low = _mm_blendv_epi8(b_low, a_low, cond_low);
+                __m128i res_high = _mm_blendv_epi8(b_high, a_high, cond_high);
+                XSIMD_RETURN_MERGED_SSE(res_low, res_high);
 #endif
+            }
+        };
     }
 
     inline batch<int32_t, 8> operator<<(const batch<int32_t, 8>& lhs, int32_t rhs)
